@@ -1,7 +1,7 @@
 /*
  * acl.h - Define the ACL interface
  *
- * Copyright (C) 2013 - 2016, Max Lv <max.c.lv@gmail.com>
+ * Copyright (C) 2013 - 2018, Max Lv <max.c.lv@gmail.com>
  *
  * This file is part of the shadowsocks-libev.
  *
@@ -26,12 +26,28 @@
 #define BLACK_LIST 0
 #define WHITE_LIST 1
 
-int init_acl(const char *path, int mode);
-void free_acl(void);
+#define MAX_TRIES  256
+#define MALICIOUS  8
+#define SUSPICIOUS 4
+#define BAD        2
+#define MALFORMED  1
 
-int acl_get_mode(void);
-int acl_match_ip(const char *ip);
+int init_acl(const char *path);
+void free_acl(void);
+void clear_block_list(void);
+
+int acl_match_host(const char *ip);
 int acl_add_ip(const char *ip);
 int acl_remove_ip(const char *ip);
 
+int get_acl_mode(void);
+
+void init_block_list();
+void free_block_list();
+int check_block_list(char *addr);
+int update_block_list(char *addr, int err_level);
+int remove_from_block_list(char *addr);
+
+int outbound_block_match_host(const char *host);
+
 #endif // _ACL_H
