@@ -1,7 +1,7 @@
 /*
  * server.h - Define shadowsocks server's buffers and callbacks
  *
- * Copyright (C) 2013 - 2016, Max Lv <max.c.lv@gmail.com>
+ * Copyright (C) 2013 - 2018, Max Lv <max.c.lv@gmail.com>
  *
  * This file is part of the shadowsocks-libev.
  *
@@ -23,10 +23,15 @@
 #ifndef _MANAGER_H
 #define _MANAGER_H
 
-#include <ev.h>
 #include <time.h>
 #include <libcork/ds.h>
 
+#ifdef HAVE_LIBEV_EV_H
+#include <libev/ev.h>
+#else
+#include <ev.h>
+#endif
+
 #include "jconf.h"
 
 #include "common.h"
@@ -35,25 +40,38 @@ struct manager_ctx {
     ev_io io;
     int fd;
     int fast_open;
+    int reuse_port;
     int verbose;
     int mode;
-    int auth;
     char *password;
+    char *key;
     char *timeout;
     char *method;
     char *iface;
     char *acl;
     char *user;
+    char *plugin;
+    char *plugin_opts;
     char *manager_address;
     char **hosts;
     int host_num;
     char **nameservers;
     int nameserver_num;
+    int mtu;
+    int ipv6first;
+#ifdef HAVE_SETRLIMIT
+    int nofile;
+#endif
 };
 
 struct server {
     char port[8];
     char password[128];
+    char fast_open[8];
+    char *mode;
+    char *method;
+    char *plugin;
+    char *plugin_opts;
     uint64_t traffic;
 };
 
